@@ -44,7 +44,6 @@ gjs_throw_valist(JSContext       *context,
                     const char      *format,
                     va_list          args)
 {
-    JSString* jstr;
     char *s;
     jsval retval;
     jsval argv[1];
@@ -56,6 +55,8 @@ gjs_throw_valist(JSContext       *context,
 
     s = g_strdup_vprintf(format, args);
 
+    JS_BeginRequest(context);
+
     if (JS_IsExceptionPending(context)) {
         /* Often it's unclear whether a given jsapi.h function
          * will throw an exception, so we will throw ourselves
@@ -76,8 +77,7 @@ gjs_throw_valist(JSContext       *context,
 
     JS_EnterLocalRootScope(context);
 
-    jstr = JS_NewStringCopyZ(context, s);
-    if (jstr == NULL) {
+    if (!gjs_string_from_utf8(context, s, -1, &argv[0])) {
         JS_ReportError(context, "Failed to copy exception string");
         goto out;
     }
@@ -105,7 +105,6 @@ gjs_throw_valist(JSContext       *context,
     }
 
     retval = JSVAL_VOID;
-    argv[0] = STRING_TO_JSVAL(jstr);
 
     /* note the return value is whether function succeeded, which it shouldn't, since it
      * throws...
@@ -141,6 +140,8 @@ gjs_throw_valist(JSContext       *context,
                        s);
     }
     g_free(s);
+
+    JS_EndRequest(context);
 }
 
 /* Throws an exception, like "throw new Error(message)"
@@ -164,7 +165,7 @@ gjs_throw(JSContext       *context,
 
 /**
  * gjs_throw_literal:
- * 
+ *
  * Similar to gjs_throw(), but does not treat its argument as
  * a format string.
  */
@@ -177,7 +178,7 @@ gjs_throw_literal(JSContext       *context,
 
 /**
  * gjs_throw_g_error:
- * 
+ *
  * Convert a GError into a JavaScript Exception, and
  * frees the GError.  Like gjs_throw(), will not overwrite
  * an already pending exception.
@@ -215,6 +216,9 @@ gjstest_test_func_gjs_jsapi_util_error_throw(void)
      */
     runtime = JS_NewRuntime(1024*1024 /* max bytes */);
     context = JS_NewContext(runtime, 8192);
+
+    JS_BeginRequest(context);
+
     global = JS_NewObject(context, NULL, NULL, NULL);
     JS_SetGlobalObject(context, global);
     JS_InitStandardClasses(context, global);
@@ -269,6 +273,8 @@ gjstest_test_func_gjs_jsapi_util_error_throw(void)
 
     JS_RemoveRoot(context, &previous);
 
+    JS_EndRequest(context);
+
     JS_DestroyContext(context);
     JS_DestroyRuntime(runtime);
     JS_ShutDown();
