@@ -26,47 +26,54 @@
 #include <string.h>
 
 #include "jsapi-util.h"
+#include "compat.h"
 
-JSBool
-gjs_string_to_utf8(JSContext  *context,
-                   const jsval string_val,
-                   char      **utf8_string_p)
+gboolean
+gjs_try_string_to_utf8 (JSContext  *context,
+                        const jsval string_val,
+                        char      **utf8_string_p,
+                        GError    **error)
 {
     jschar *s;
     size_t s_length;
     char *utf8_string;
     long read_items;
     long utf8_length;
-    GError *error;
+    GError *convert_error = NULL;
+
+    JS_BeginRequest(context);
 
     if (!JSVAL_IS_STRING(string_val)) {
-        gjs_throw(context,
-                     "Object is not a string, cannot convert to UTF-8");
-        return JS_FALSE;
+        g_set_error_literal(error, GJS_UTIL_ERROR, GJS_UTIL_ERROR_ARGUMENT_TYPE_MISMATCH,
+                            "Object is not a string, cannot convert to UTF-8");
+        JS_EndRequest(context);
+        return FALSE;
     }
 
     s = JS_GetStringChars(JSVAL_TO_STRING(string_val));
     s_length = JS_GetStringLength(JSVAL_TO_STRING(string_val));
 
-    error = NULL;
     utf8_string = g_utf16_to_utf8(s,
                                   (glong)s_length,
                                   &read_items, &utf8_length,
-                                  &error);
+                                  &convert_error);
+
+    /* ENDING REQUEST - no JSAPI after this point */
+    JS_EndRequest(context);
 
     if (!utf8_string) {
-        gjs_throw(context,
-                  "Failed to convert JS string to "
-                  "UTF-8: %s",
-                  error->message);
-        g_error_free(error);
-        return JS_FALSE;
+        g_set_error(error, GJS_UTIL_ERROR, GJS_UTIL_ERROR_ARGUMENT_INVALID,
+                    "Failed to convert JS string to UTF-8: %s",
+                    convert_error->message);
+        g_error_free(convert_error);
+        return FALSE;
     }
 
     if ((size_t)read_items != s_length) {
-        gjs_throw(context, "JS string contains embedded NULs");
+        g_set_error_literal(error, GJS_UTIL_ERROR, GJS_UTIL_ERROR_ARGUMENT_INVALID,
+                            "JS string contains embedded NULs");
         g_free(utf8_string);
-        return JS_FALSE;
+        return FALSE;
     }
 
     /* Our assumption is that the string is being converted to UTF-8
@@ -79,19 +86,35 @@ gjs_string_to_utf8(JSContext  *context,
      * case of all-ASCII.
      */
     if (!g_utf8_validate (utf8_string, utf8_length, NULL)) {
-        gjs_throw(context, "JS string contains invalid Unicode characters");
+        g_set_error_literal(error, GJS_UTIL_ERROR, GJS_UTIL_ERROR_ARGUMENT_INVALID,
+                            "JS string contains invalid Unicode characters");
         g_free(utf8_string);
-        return JS_FALSE;
+        return FALSE;
     }
 
     *utf8_string_p = utf8_string;
-    return JS_TRUE;
+    return TRUE;
+}
+
+JSBool
+gjs_string_to_utf8 (JSContext  *context,
+                    const jsval string_val,
+                    char      **utf8_string_p)
+{
+  GError *error = NULL;
+
+  if (!gjs_try_string_to_utf8(context, string_val, utf8_string_p, &error))
+    {
+      gjs_throw_g_error(context, error);
+      return JS_FALSE;
+    }
+  return JS_TRUE;
 }
 
 JSBool
 gjs_string_from_utf8(JSContext  *context,
                      const char *utf8_string,
-                     gsize       n_bytes,
+                     gssize      n_bytes,
                      jsval      *value_p)
 {
     jschar *u16_string;
@@ -119,55 +142,72 @@ gjs_string_from_utf8(JSContext  *context,
         return JS_FALSE;
     }
 
+    JS_BeginRequest(context);
+
     s = JS_NewUCStringCopyN(context,
                             (jschar*)u16_string,
                             u16_string_length);
     g_free(u16_string);
 
-    if (!s)
+    if (!s) {
+        JS_EndRequest(context);
         return JS_FALSE;
+    }
 
     *value_p = STRING_TO_JSVAL(s);
+    JS_EndRequest(context);
     return JS_TRUE;
 }
 
+gboolean
+gjs_try_string_to_filename(JSContext    *context,
+                           const jsval   filename_val,
+                           char        **filename_string_p,
+                           GError      **error)
+{
+  gchar *tmp, *filename_string;
+
+  /* gjs_string_to_filename verifies that filename_val is a string */
+
+  if (!gjs_try_string_to_utf8(context, filename_val, &tmp, error)) {
+      /* exception already set */
+      return JS_FALSE;
+  }
+
+  error = NULL;
+  filename_string = g_filename_from_utf8(tmp, -1, NULL, NULL, error);
+  if (!filename_string) {
+    g_free(tmp);
+    return FALSE;
+  }
+
+  *filename_string_p = filename_string;
+
+  g_free(tmp);
+  return TRUE;
+}
+
 JSBool
 gjs_string_to_filename(JSContext    *context,
                        const jsval   filename_val,
                        char        **filename_string_p)
 {
-    GError *error;
-    gchar *tmp, *filename_string;
+  GError *error = NULL;
 
-    /* gjs_string_to_filename verifies that filename_val is a string */
-
-    if (!gjs_string_to_utf8(context, filename_val, &tmp)) {
-        /* exception already set */
-        return JS_FALSE;
-    }
-    
-    error = NULL;
-    filename_string = g_filename_from_utf8(tmp, -1, NULL, NULL, &error);
-    if (error) {
-        gjs_throw(context,
-                  "Could not convert filename '%s' to UTF8: '%s'",
-                  tmp,
-                  error->message);
-        g_error_free(error);
-        g_free(tmp);
-        return JS_FALSE;
-    }
-
-    *filename_string_p = filename_string;
-    
-    g_free(tmp);
-    return JS_TRUE;
+  if (!gjs_try_string_to_filename(context, filename_val, filename_string_p, &error)) {
+      gjs_throw(context,
+                "Could not convert filename to UTF8: '%s'",
+                error->message);
+      g_error_free(error);
+      return JS_FALSE;
+  }
+  return JS_TRUE;
 }
 
 JSBool
 gjs_string_from_filename(JSContext  *context,
                          const char *filename_string,
-                         gsize       n_bytes,
+                         gssize      n_bytes,
                          jsval      *value_p)
 {
     gsize written;
@@ -186,7 +226,7 @@ gjs_string_from_filename(JSContext  *context,
         g_free(utf8_string);
         return JS_FALSE;
     }
-    
+
     if (!gjs_string_from_utf8(context, utf8_string, written, value_p))
         return JS_FALSE;
 
@@ -272,14 +312,18 @@ gjs_string_get_binary_data(JSContext       *context,
 {
     char *js_data;
 
+    JS_BeginRequest(context);
+
     if (!JSVAL_IS_STRING(value)) {
         gjs_throw(context,
                   "Value is not a string, can't return binary data from it");
         return JS_FALSE;
     }
 
-    if (throw_if_binary_strings_broken(context))
+    if (throw_if_binary_strings_broken(context)) {
+        JS_EndRequest(context);
         return JS_FALSE;
+    }
 
     js_data = JS_GetStringBytes(JSVAL_TO_STRING(value));
     /* GetStringLength returns number of 16-bit jschar;
@@ -288,6 +332,8 @@ gjs_string_get_binary_data(JSContext       *context,
     *len_p = JS_GetStringLength(JSVAL_TO_STRING(value));
     *data_p = g_memdup(js_data, *len_p);
 
+    JS_EndRequest(context);
+
     return JS_TRUE;
 }
 
@@ -310,8 +356,12 @@ gjs_string_from_binary_data(JSContext       *context,
 {
     JSString *s;
 
-    if (throw_if_binary_strings_broken(context))
+    JS_BeginRequest(context);
+
+    if (throw_if_binary_strings_broken(context)) {
+        JS_EndRequest(context);
         return JS_FALSE;
+    }
 
     /* store each byte in a 16-bit jschar so all high bytes are 0;
      * we can't put two bytes per jschar because then we'd lose
@@ -321,10 +371,12 @@ gjs_string_from_binary_data(JSContext       *context,
     if (s == NULL) {
         /* gjs_throw() does nothing if exception already set */
         gjs_throw(context, "Failed to allocate binary string");
+        JS_EndRequest(context);
         return JS_FALSE;
     }
     *value_p = STRING_TO_JSVAL(s);
 
+    JS_EndRequest(context);
     return JS_TRUE;
 }
 
@@ -349,9 +401,12 @@ gjs_string_get_uint16_data(JSContext       *context,
 {
     jschar *js_data;
 
+    JS_BeginRequest(context);
+
     if (!JSVAL_IS_STRING(value)) {
         gjs_throw(context,
                   "Value is not a string, can't return binary data from it");
+        JS_EndRequest(context);
         return JS_FALSE;
     }
 
@@ -359,6 +414,7 @@ gjs_string_get_uint16_data(JSContext       *context,
     *len_p = JS_GetStringLength(JSVAL_TO_STRING(value));
     *data_p = g_memdup(js_data, sizeof(*js_data)*(*len_p));
 
+    JS_EndRequest(context);
     return JS_TRUE;
 }
 
@@ -410,6 +466,7 @@ gjstest_test_func_gjs_jsapi_util_string_js_string_utf8(void)
 
     runtime = JS_NewRuntime(1024*1024 /* max bytes */);
     context = JS_NewContext(runtime, 8192);
+    JS_BeginRequest(context);
     global = JS_NewObject(context, NULL, NULL, NULL);
     JS_SetGlobalObject(context, global);
     JS_InitStandardClasses(context, global);
@@ -421,6 +478,7 @@ gjstest_test_func_gjs_jsapi_util_string_js_string_utf8(void)
     g_assert(JSVAL_IS_STRING(js_string));
     g_assert(gjs_string_to_utf8(context, js_string, &utf8_result) == JS_TRUE);
 
+    JS_EndRequest(context);
     JS_DestroyContext(context);
     JS_DestroyRuntime(runtime);
 
@@ -441,6 +499,7 @@ gjstest_test_func_gjs_jsapi_util_string_get_ascii(void)
 
     runtime = JS_NewRuntime(1024*1024 /* max bytes */);
     context = JS_NewContext(runtime, 8192);
+    JS_BeginRequest(context);
     global = JS_NewObject(context, NULL, NULL, NULL);
     JS_SetGlobalObject(context, global);
     JS_InitStandardClasses(context, global);
@@ -453,6 +512,7 @@ gjstest_test_func_gjs_jsapi_util_string_get_ascii(void)
     g_assert(gjs_string_get_ascii_checked(context, void_value) == NULL);
     g_assert(JS_IsExceptionPending(context));
 
+    JS_EndRequest(context);
     JS_DestroyContext(context);
     JS_DestroyRuntime(runtime);
 }
@@ -475,6 +535,7 @@ gjstest_test_func_gjs_jsapi_util_string_get_binary(void)
 
     runtime = JS_NewRuntime(1024*1024 /* max bytes */);
     context = JS_NewContext(runtime, 8192);
+    JS_BeginRequest(context);
     global = JS_NewObject(context, NULL, NULL, NULL);
     JS_SetGlobalObject(context, global);
     JS_InitStandardClasses(context, global);
@@ -521,6 +582,7 @@ gjstest_test_func_gjs_jsapi_util_string_get_binary(void)
                                         &data, &len));
     g_assert(JS_IsExceptionPending(context));
 
+    JS_EndRequest(context);
     JS_DestroyContext(context);
     JS_DestroyRuntime(runtime);
 }
