@@ -323,7 +323,7 @@ invoke_js_from_dbus(JSContext   *context,
     rval = JSVAL_VOID;
     JS_AddRoot(context, &rval);
 
-    gjs_js_set_current_message(method_call);
+    gjs_js_push_current_message(method_call);
     if (!gjs_call_function_value(context,
                                  this_obj,
                                  OBJECT_TO_JSVAL(method_obj),
@@ -369,7 +369,7 @@ invoke_js_from_dbus(JSContext   *context,
     gjs_rooted_array_free(context, values, TRUE);
     JS_RemoveRoot(context, &rval);
 
-   gjs_js_set_current_message(NULL);
+    gjs_js_pop_current_message();
 
     if (reply)
         gjs_debug(GJS_DEBUG_DBUS, "Sending %s reply to dbus method %s",
@@ -1852,24 +1852,30 @@ gjs_js_define_dbus_exports(JSContext      *context,
 {
     JSObject *exports;
     JSContext *load_context;
+    JSBool success;
 
+    success = JS_FALSE;
     load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
+    JS_BeginRequest(load_context);
 
     exports = exports_new(load_context, which_bus);
     if (exports == NULL) {
         gjs_move_exception(load_context, context);
-        return JS_FALSE;
+        goto fail;
     }
 
     if (!add_connect_funcs(context, exports, which_bus))
-        return JS_FALSE;
+        goto fail;
 
     if (!JS_DefineProperty(context, in_object,
                            "exports",
                            OBJECT_TO_JSVAL(exports),
                            NULL, NULL,
                            GJS_MODULE_PROP_FLAGS))
-        return JS_FALSE;
+        goto fail;
 
-    return JS_TRUE;
+    success = JS_TRUE;
+ fail:
+    JS_EndRequest(load_context);
+    return success;
 }
